<?php

namespace Drupal\cforge;

use Drupal\mcapi\Entity\Currency;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Core\Url;
use Drupal\Core\Render\Element;
use Drupal\Core\Logger\LoggerChannel;
Use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Utility\LinkGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ExtensionDiscovery;


ini_set('max_execution_time', 150);

/**
 * Configuration options for each flavour.
 *
 */
class Setup extends FormBase {

  /**
   * @var ModuleInstallerInterface
   */
  private $installer;

  /**
   * @var EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * @var LinkGeneratorInterface
   */
  protected $linkGenerator;

  /**
   * @var LoggerChannel
   */
  protected $logger;

  /**
   * Constructor.
   */
  public function __construct(ModuleInstallerInterface $module_installer, LinkGeneratorInterface $link_generator, EntityTypeManagerInterface $entity_type_manager, LoggerChannel $logger_channel) {
    $this->installer = $module_installer;
    $this->linkGenerator = $link_generator;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_channel;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('link_generator'),
      $container->get('entity_type.manager'),
      $container->get('logger.channel.cforge')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_setup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    //$form_state->loadInclude('cforge','install');

    foreach (system_rebuild_module_data() as $module_name => $extension) {
      $info = $extension->info;
      if ($info['package'] == 'Cforge features' and empty($info['hidden'])) {
        if (!\Drupal::moduleHandler()->moduleExists($module_name)) {
          $feature_names[$module_name] = $this->t($info['name']);
          $feature_descriptions[$module_name] = $this->t($info['description']);
        }
      }
    }

    foreach ($feature_descriptions as $name => $description) {
      $form['features'][$name]['#description'] = $description;
      $form['features'][$name]['#states']['checked'][':input[name="demo"]']['checked'] = TRUE;
    }
    $form['features'] = [
      // Hope that yields the current language.
      '#title' => (string) t('Hamlets features'),
      '#type' => 'checkboxes',
      '#options' => $feature_names,
      // Everything enabled by default.
      '#default_value' => array_keys($feature_names),
      '#weight' => 1,
    ];

    $form['data_content'] = [
      '#title' => $this->t('Prepare the site with'),
      '#type' => 'radios',
      '#options' => [
        'none' => t('No content'),
        'import' => $this->t('CSV Import Tools'),
        'upgrade' => $this->t('Upgrade modules'),
      ],
      '#default_value'  => 'none',
      '#weight' => 3,
      '#required' => TRUE,
    ];

    $alt_login_form = \Drupal::formBuilder()->getForm('Drupal\alt_login\Settings');
    $form['aliases'] = [
      '#title' => $alt_login_form['aliases']['#title'],
      '#type' => 'select',
      '#options' => $alt_login_form['aliases']['#options'],
      '#default_value' => 'address_name',
      '#weight' => 5
    ];

    $form['default_categories'] = array(
      '#title' => $this->t('Preload ad categories'),
      '#description' => $this->t('Can be modified later'),
      '#type' => 'radios',
      '#options' => [
        0 => $this->t('- None -'),
        'en' => 'English LETS',
        'fr' => 'French',
        'be' => 'SEL Belgique, (2 Tier]'
      ],
      '#default_value' => \Drupal::languageManager()->getDefaultLanguage()->getId(),
      '#weight' => 9,
      '#states' => [
        'visible' => [
          ':input[name=data_content]' => ['value' => 'none'],
        ],
      ],
    );

    $form['currency_type'] = [
      '#title' => $this->t('Main currency type'),
      '#type' => 'radios',
      '#options' => [
        'hours' => $this->t('Hours'),
        'other' => $this->t('Other'),
      ],
      '#default_value' => 'hours',
      '#required' => TRUE,
      '#weight' => 6,
    ];
    $form['currency_name'] = [
      '#title' => $this->t('Currency name'),
      '#description' => $this->t('Use the plural'),
      '#type' => 'textfield',
      '#placeholder' => $this->t('Units'),
      '#weight' => 8,
      '#states' => [
        'visible' => [
          ':input[name="currency_type"]' => ['value' => 'other'],
        ],
      ],
    ];

    $form['direction'] = [
      '#title' => $this->t('Default transaction direction'),
      '#type' => 'radios',
      '#options' => [
        'credit' => $this->t('Credit (push payment)'),
        'bill' => $this->t('Bill (pull payment)'),
        'both' => $this->t('Both (may be confusing for some users)')
      ],
      '#default_value' => 'credit',
      '#weight' => 10,
    ];

    $form['devel'] = [
      '#title' => t('Devel modules'),
      '#type' => 'checkbox',
      '#weight'=> 15
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'submit',
      '#weight' => 20,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $discovery = new ExtensionDiscovery(\Drupal::root());
    $available_modules = array_keys($discovery->scan('module'));
    foreach($available_modules as $module) {
      if (substr($module, 0, 7) == 'cforge_') {
        module_set_weight($module, 101);
      }
    }

    if ($form_state->getValue('direction') == 'credit') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.bill')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }
    elseif ($form_state->getValue('direction') == 'bill') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.credit')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }

    if ($form_state->getValue('default_categories')) {
      $terms = $this->entityTypeManager
        ->getStorage('taxonomy_term')
        ->loadByProperties(['vid' => SMALLAD_CATEGORIES]);
      // Delete any existing terms in the vocab (one is installed by default)
      foreach ($terms as $term) {
        $term->delete();
      }
      $vocab = Vocabulary::load(SMALLAD_CATEGORIES);
      // Now import the terms into the vocab.
      $get_terms = 'vocab_'.$form_state->getValue('default_categories');
      $this->vocabImportRecursive($vocab, $this->$get_terms());
      $vocab->save();
    }

    // Set up the currency by deleting the others.
    if ($c = Currency::load('veur')) {
      $c->delete();
    }
    if ($form_state->getValue('currency_type') == 'hours') {
      if ($c = Currency::load('cc')) {
        $c->delete();
      }
    }
    if ($form_state->getValue('currency_type') == 'cc') {
      if ($c = Currency::load('hhrs')) {
        $c->delete();
      }
    }
    if ($currs = Currency::loadMultiple()) {
      $curr = reset($currs);
      $curr->name = $form_state->getValue('currency_name') ?: 'Units';
      $symbol = strtoupper(substr($curr->name, 0, 1));
      $curr->format = ["<strong>$symbol</strong>", '9999'];
      $curr->save();
    }

    $modules = array_filter($form_state->getValue('features'));

    if ($form_state->getValue('devel')) {
      $modules = array_merge($modules, [
        'devel_php',
        'dblog',
        'field_ui',
        'views_ui',
        'contextual',
        'admin_toolbar_tools',
        'config'
      ]);
    }
    switch ($form_state->getValue('data_content')) {
      case 'import':
        $modules[] = 'cforge_import';
        break;
      case 'upgrade':
        $modules = array_merge($modules, [
          'migrate',
          'migrate_drupal',
          'migrate_drupal_ui',
          'migrate_plus',
          'migrate_tools',
          'migrate_upgrade',
          'options'
        ]);
        break;
      case 'none':
        cf_neighbourhoods_create(['Centre']);
    }

    $alt_login_conf = $this->configfactory()->getEditable('alt_login.settings')
      ->set('display_anon', 'Member [user:uid]')
      ->set('display', '[user:address:given_name] [user:address:family_name]')
      ->set('aliases', [$form_state->getvalue('aliases')]);
    if ($form_state->getvalue('aliases') == 'uid') {
      $alt_login_conf
        ->set('display', '[user:address:given_name] [user:address:family_name] ([user:uid])');
    }
    $alt_login_conf->save();
    $form_state->setRedirectUrl(
      Url::fromUserInput($this->config('cforge.settings')->get('member_frontpage'))
    );

    if (empty($modules)) {
      return;
    }

    $this->installer->install($modules);
    $this->installer->uninstall(['default_content']);

    return;
    $batch_builder = (new BatchBuilder())
      ->setTitle(t('Installing features'))
      ->setProgressMessage(t('Processed @current out of @total.'))
      ->addOperation([$this->installer, 'install'], [$modules])
      ->addOperation([$this->installer, 'uninstall'], [['default_content']]);
    batch_set($batch_builder->toArray());
  }

  /**
   * Import the terms in a vocabulary or a level of a vocab.
   */
  private function vocabImportRecursive(Vocabulary $vocab, array $termArray, $parent = 0) {
    $w = $hierarchy = 0;
    foreach (Element::Children($termArray) as $name) {
      $props = [
        'name' => $name,
        'weight' => $w++,
        'vid' => $vocab->id(),
        'parent' => $parent,
        'format' => 'plain_text',
      ];
      $term = Term::Create($props);
      $term->save();
      if ($termArray['#children']) {
        // @todo switch over these lines on 8.2.0
        //$vocab->setHierarchy(VocabularyInterface::HIERARCHY_SINGLE);
        $vocab->setHierarchy(TAXONOMY_HIERARCHY_SINGLE);
        $this->vocabImportRecursive($vocab, $termArray[$name], $term->tid);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  private function vocab_en() {
    return [
      '#children' => FALSE,
      'Arts & Culture' => [],
      'Business Services & Clerical' => [],
      'Clothing' => [],
      'Computing & Electronics' => [],
      'Education & Language' => [],
      'Food' => [],
      'Health & Wellness' => [],
      'House & garden' => [],
      'Sports & Leisure' => [],
      'Skills & DIY' => [],
      'Transport' => [],
      'Miscellaneous' => []
    ];
  }

  /**
   * {@inheritdoc}
   */
  private function vocab_fr() {
    return [
      '#children' => FALSE,
      'Administration' => [],
      'Alimentation' => [],
      'Artisanat & Bricolage' => [],
      'Arts & Culture' => [],
      'Cours & Langues' => [],
      'Informatique & Eléctro' => [],
      'Maison & Jardin' => [],
      'Mobilité' => [],
      'Santé & Soins' => [],
      'Sports & Evasion' => [],
      'Vêtements plus' => [],
      'Divers' => [],
    ];
  }
  private function vocab_be() {
    return [
      '#children' => TRUE,
      'A Ménage / Entretien maison' => [
        '#children' => FALSE,
        "Nettoyage/ produits d'entretien" => [],
        'Lessive, repassage' => [],
        'Couture, tricot' => [],
        'Rangement' => [],
        'Ménage - Divers' => [],
      ],
      'B Travaux maisons' => [
        '#children' => FALSE,
        'Travaux lourds' => [],
        'Petits travaux divers' => [],
        'Peinture, tapissage, décoration' => [],
        'Electricité' => [],
        'Plomberie, chauffage' => [],
        'Carrelage, plafonnage, maçonnerie' => [],
        'Menuiserie, planchers, meubles' => [],
        "Isolation, économies d'énergie" => [],
        "Travaux - Divers" => [],
      ],
      'C Enfants / Ados' => [
        '#children' => FALSE,
        'Grossesse et bébé' => [],
        'Baby sitting' => [],
        'Stages, animations créatives' => [],
        'Aide scolaire' => [],
        'Jeux et jouets' => [],
        'Enfants - Divers' => [],
      ],
      'D Alimentation / Gastronomie' => [
        '#children' => FALSE,
        'Nettoyage fruits et légumes' => [],
        'Boissons, soupes, sauces' => [],
        'Plats' => [],
        'Desserts' => [],
        "Tables d'hôtes" => [],
        'Cours, conseils, recettes' => [],
        'Alimentation - Divers' => [],
      ],
      'E Santé / Bien-être / Accompagnement' => [
        '#children' => FALSE,
        'Coiffure, esthétique visage et corps' => [],
        'Massage, thérapies, produits naturels' => [],
        'Personnes âgées, malades, immobilisées' => [],
        "Accompagnement, coaching et conseils 'psy'" => [],
        'Santé - Divers' => [],
      ],
      'F Administration / Gestion' => [
        '#children' => FALSE,
        'Classement de papiers, de dossiers' => [],
        'Dactylographie, mise en page' => [],
        'Courriers, démarches administratives' => [],
        'Assurances, questions juridiques' => [],
        'Fiscalité, pension, chômage' => [],
        'Administration - Divers' => [],
      ],
      'G Cours / Formations / Conseils' => [
        '#children' => FALSE,
        'Langues, conversation, traduction' => [],
        'Ecriture, rédaction, orthographe' => [],
        "Recherche d'emploi" => [],
        'Gestion et animation de groupes' => [],
        'Cours - Divers' => [],
      ],
      'H Jardin / Animaux / Nature' => [
        '#children' => FALSE,
        'Jardin : conception, entretien, conseils' => [],
        'Culture potagère, vergers' => [],
        'Récolte de fruits, légumes, plantes' => [],
        "Taille d'arbres et arbustes" => [],
        'Animaux (domestiques -élevage]' => [],
        'Biodiversité, nichoirs, découverte nature' => [],
        'Engrais, semences, plants, produits divers' => [],
        'Jardin et animaux - Divers' => [],
      ],
      'I Arts / Culture / Sports & loisirs' => [
        '#children' => FALSE,
        'Artisanat, peinture, dessin, sculpture' => [],
        'Musique, chant, danse' => [],
        'Photo, vidéo, cinema' => [],
        'Animations, spectacles' => [],
        'Fêtes : conseils, organisation, aide' => [],
        'Promenades, excursions' => [],
        "Jeux (de société, d'extérieur]" => [],
        'Sports' => [],
        'Culture et sport - Divers' => [],
      ],
      'J Transports / Voyages /Hébergement' => [
        '#children' => FALSE,
        'Courses' => [],
        'Covoiturage et transport' => [],
        'Déménagement' => [],
        'Auto-moto-vélo' => [],
        'Voyages, randonnées' => [],
        'Hébergement, échange maisons' => [],
        'Gardiennage maison' => [],
        'Transport - Divers' => [],
      ],
      'K Informatique /Électroménager' => [
        '#children' => FALSE,
        'Dépannage informatique' => [],
        'Réparation électroménager' => [],
        'Formations Internet / email' => [],
        'Formations logiciels' => [],
        'Informatique, électroménager - Divers' => [],
      ],
      'Z Divers (non classées]' => [
        '#children' => FALSE,
      ]
    ];
  }

}
